.TH make-olwm-menu L "1.1" "Auburn University"
\" @(#)make-olwm-menu.man	1.1	04 Feb 1993	Auburn University Engineering
.SH NAME
make-olwm-menu \- create customized OpenLook window manager menu
.SH SYNOPSIS
.B make-olwm-menu
.SH DESCRIPTION
\fBmake-olwm-menu\fP creates a customized OpenLook window manager
menu by including menu information found in the \fBModules\fP package.
.LP
The constructed menu is placed in the file \fI$HOME/.openwin-menu.\fP
.BR olwm
version 3 or later will automatically reread the \fI$HOME/.openwin-menu.\fP
If \fI$HOME/.openwin-menu\fP did not previously exist, then the window
manager may have to be restarted in order to recognize the user's menu.
When \fBmake-olwm-menu\fP has successfully created a new menu, the user's
old menu will be moved to \fI$HOME/.openwin-menu.bak\fP and the new menu
installed.
.LP
\fBmake-olwm-menu\fP first looks for a \fI$HOME/.personal-menu\fP.  If
one is found, it is included as a "Personal" submenu.
.LP
Next, every \fImodulefile\fP that is loaded is used to search for
submenus to be placed under the "Applications" submenu.
Each \fImodulefile\fP may be accompanied by \fImodulefile.menu\fP and
\fImodulefile.title\fP files.  The \fI.menu\fP file contains the application
submenu commands for the \fImodulefile.\fP  The \fI.title\fP file contains
a title for the application submenu.
.LP
Finally, each \fImodulefile\fP is searched for a \fImodulefile.base-menu\fP
file.  The \fI.base-menu\fP file is expected to be associated with the
window system.  The file is concatenated to the constructed menu.  Only
one \fI.base-menu\fP file will be used.
.SH ENVIRONMENT
.TP 2.5i
_LMFILES_
colon separated list of loaded modulefiles
.SH FILES
.TP 2.5i
$HOME/.personal-menu
optional personal menu which will be included in the constructed menu.
.TP 2.5i
$HOME/.openwin-menu
menu used by
.BR olwm.
If this file does not exist, then the system default menu will be used.
.TP 2.5i
$HOME/.openwin-menu.bak
backup copy of the previous menu.
.TP 2.5i
\fImodulefile\fP.title
file containing title for \fImodulefile's\fP application submenu.
.TP 2.5i
\fImodulefile\fP.menu
file containing submenu.
.TP 2.5i
\fImodulefile\fP.base-menu
file containing menu information for a base window system.
.SH "SEE ALSO"
.BR olwm(1),
.BR module(1)
.SH NOTES
If the constructed menu is bad, the window manager will revert to a
default menu.  In this case, the constructed menu should be fixed
before attempting the reread the menu.

